/* 
 *  @TITLE "test2a.c"
 *
 *  This file contains a test driver for the modularized diskdevice
 *  module. This one reads a Unix file onto the disk, and then reads 
 *  it back out again.
 *
 *     Song Bac Toh
 */
/* $Id: test2a.c,v 2.0 94/07/17 18:35:25 dfk Exp $ */


/* This main() function is only used when the program is not attached
   to proteus.  So if PROTEUS is defined, this file will skip the 
   preprocessor completely.*/

#include "diskdevice.h"
#include "modularize.h"
#include "queue.h"
#define BQSIZE    15		/* size of bus queue */
#define MSECperUSEC     ( 1.0 / 1000.0 )
#define TICSperUSEC     (TICSperMSEC * MSECperUSEC)
#define STUFFY          "I am a stuffed string"
#define DISKFILE        "vdisk"

extern void srandom(int seed);
Heap heap;			/* the event heap */

extern int copy2VDisk(char *fileName, int diskID);
extern int copyfromVDisk(int diskID, char *fileName);


void main(int argc, char **argv)
{
  ulong disk;
  char busString[] = "bus queue";
  NCQUEUE *busWaitq;
  int busOwner;
  int busId;

  /* ------------ needs 3 parameters ---------- */
  if (argc != 3)
    {
      fprintf(stderr, "syntax: %s infileName outfileName\n", argv[0]);
      exit(FATAL);
    }
  /* ------------------------------------------------ */

  srandom(time((time_t *)0));	    /* plant a random seed */
  InitTime();		            /* initialize the global clock */

  /* In this test driver, I only use 1 disk, ie disk 1 */
  busOwner = BUS_FREE;
  busWaitq = MakeQueue_noncyc(BQSIZE, busString);

  /* Each disk has a bus; but since I only use disk 1, I only initialize
     a disk and its bus */
  disk = 1;
  busId = 1;
  /* the fifth parameter can either be DISKFILE or NULL */
  DiskDeviceInit(disk, busId, &busOwner, busWaitq, DISKFILE);

  copy2VDisk(argv[1], 1);
  DiskDeviceSync(1);
  copyfromVDisk(1, argv[2]);

  /* Done with the disk, reset the info and shut it down */
  DiskDeviceDone(disk);  
}
